@@ -5,38 +5,32 @@ from typing import Optional, List, Dict
 import requests
 
 from .provider import Provider
-from ..constants import HTTP_REQUEST_TIMEOUT
-from ..keys import PublicKey
+from ..constants import Chain
 from ..script.type import P2pkScriptType
 
 
 class SensibleQuery(Provider):
 
-    def __init__(self):
+    def __init__(self, headers: Optional[Dict] = None, timeout: Optional[int] = None):
+        super().__init__(Chain.MAIN, headers, timeout)
         self.url: str = 'https://api.sensiblequery.com'
-        self.headers: Dict = {'Content-Type': 'application/json', 'Accept': 'application/json', }
-        self.timeout: int = HTTP_REQUEST_TIMEOUT
-
-    def get(self, **kwargs) -> Dict:
-        r = requests.get(kwargs['url'], headers=self.headers, params=kwargs.get('params'), timeout=self.timeout)
-        r.raise_for_status()
-        return r.json()['data']
 
     def get_unspents(self, **kwargs) -> List[Dict]:
         """
         P2PKH and P2PK unspents
         """
         with suppress(Exception):
-            public_key: PublicKey = kwargs.get('public_key') or kwargs.get('private_keys')[0].public_key()
-            address: str = kwargs.get('address') or public_key.address()
-            r: Dict = self.get(url=f'{self.url}/address/{address}/utxo', params={'cursor': 0, 'size': 5120})
+            address, public_key, _ = self.parse_kwargs(**kwargs)
+            #
+            # ATTENTION
+            #   it is not recommended to use paginated queries on this API, and up to 5120 records are supported
+            #
+            r: Dict = self.get(url=f'{self.url}/address/{address}/utxo', params={'cursor': 0, 'size': 5120})['data']
             unspents: List[Dict] = []
             for item in r:  # pragma: no cover
                 unspent = {'txid': item['txid'], 'vout': item['vout'], 'satoshi': item['satoshi'], 'height': item['height']}
-                if item['scriptType'] in ['21ac', '41ac']:
+                if item['scriptType'] in ['21ac', '41ac'] and public_key:
                     # P2PK requires public key to set locking script
-                    if not public_key:
-                        continue
                     unspent['script_type'] = P2pkScriptType()
                     unspent['locking_script'] = P2pkScriptType.locking(public_key.serialize())
                 unspent.update(kwargs)
@@ -46,9 +40,8 @@ class SensibleQuery(Provider):
 
     def get_balance(self, **kwargs) -> int:
         with suppress(Exception):
-            public_key: PublicKey = kwargs.get('public_key') or kwargs.get('private_keys')[0].public_key()
-            address: str = kwargs.get('address') or public_key.address()
-            r: Dict = self.get(url=f'{self.url}/address/{address}/balance')
+            address, _, _ = self.parse_kwargs(**kwargs)
+            r: Dict = self.get(url=f'{self.url}/address/{address}/balance')['data']
             return r.get('satoshi') + r.get('pendingSatoshi')
         return 0  # pragma: no cover
 
